@@ -1,4 +1,3 @@
-/* $Id: sendmail.c,v 1.30 2000/08/11 15:08:49 fmk Exp $ */
 
 /* 
  *    PHP Sendmail for Windows.
@@ -14,10 +13,11 @@
  *  Very simple SMTP Send-mail program for sending command-line level
  *  emails and CGI-BIN form response for the Windows platform.
  *
- *  The complete wSendmail package with source code can be downloaded
- *  from http://virtual.icr.com.au:80/jgaa/cgi-bin.htm
+ *  The complete wSendmail package with source code can be located
+ *  from http://www.jgaa.com
  *
  */
+
 #include "php.h"				/*php specific */
 #include <stdio.h>
 #include <stdlib.h>
@@ -28,7 +28,7 @@
 #include <memory.h>
 #include <winbase.h>
 #include "sendmail.h"
-
+#include "php_ini.h"
 
 /*
    extern int _daylight;
@@ -45,24 +45,26 @@ static char *days[] =
 static char *months[] =
 {"Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"};
 
+#ifndef THREAD_SAFE
 char Buffer[MAIL_BUFFER_SIZE];
 
-// socket related data
+/* socket related data */
 SOCKET sc;
 WSADATA Data;
 struct hostent *adr;
 SOCKADDR_IN sock_in;
 int WinsockStarted;
-// values set by the constructor
+/* values set by the constructor */
 char *AppName;
 char MailHost[HOST_NAME_LEN];
 char LocalHost[HOST_NAME_LEN];
+#endif
 char seps[] = " ,\t\n";
-char *php_mailer = "PHP 3.0 WIN32";
+char *php_mailer = "PHP 4.0 WIN32";
 
 char *get_header(char *h, char *headers);
 
-// Error messages
+/* Error messages */
 static char *ErrorMessages[] =
 {
 	{"Success"},
@@ -84,12 +86,12 @@ static char *ErrorMessages[] =
 	{"Bad Message Return Path"},
 	{"Bad Mail Host"},
 	{"Bad Message File"},
-	{"PHP Internal error: php3.ini sendmail from variable not set!"}
+	{"PHP Internal error: php.ini sendmail from variable not set!"}
 };
 
 
 
-//********************************************************************
+/*********************************************************************
 // Name:  TSendMail
 // Input:   1) host:    Name of the mail host where the SMTP server resides
 //                      max accepted length of name = 256
@@ -101,15 +103,14 @@ static char *ErrorMessages[] =
 //                      SUCCESS otherwise.
 //
 //  See SendText() for additional args!
-//********************************************************************
+//********************************************************************/
 int TSendMail(char *host, int *error,
 			  char *headers, char *Subject, char *mailTo, char *data)
 {
 	int ret;
 	char *RPath = NULL;
-	TLS_VARS;
 
-	GLOBAL(WinsockStarted) = FALSE;
+	WinsockStarted = FALSE;
 
 	if (host == NULL) {
 		*error = BAD_MAIL_HOST;
@@ -118,16 +119,16 @@ int TSendMail(char *host, int *error,
 		*error = BAD_MAIL_HOST;
 		return BAD_MAIL_HOST;
 	} else {
-		strcpy(GLOBAL(MailHost), host);
+		strcpy(MailHost, host);
 	}
 
-	if (php3_ini.sendmail_from){
-		RPath = estrdup(php3_ini.sendmail_from);
+	if (INI_STR("sendmail_from")){
+		RPath = estrdup(INI_STR("sendmail_from"));
 		} else {
 			return 19;
 	}
 
-	// attempt to connect with mail host
+	/* attempt to connect with mail host */
 	*error = MailConnect();
 	if (*error != 0) {
 		if(RPath)efree(RPath);
@@ -150,27 +151,29 @@ int TSendMail(char *host, int *error,
 // Description: DESTRUCTOR
 // Author/Date:  jcar 20/9/96
 // History:
-//********************************************************************
+//********************************************************************/
 void TSMClose()
 {
-	TLS_VARS;
-
 	Post("QUIT\r\n");
 	Ack();
-	// to guarantee that the cleanup is not made twice and
-	// compomise the rest of the application if sockets are used
-	// elesewhere
+	/* to guarantee that the cleanup is not made twice and 
+	   compomise the rest of the application if sockets are used
+	   elesewhere 
+	*/
+
+	shutdown(sc, 0); 
+	closesocket(sc);
 }
 
 
-//********************************************************************
+/*********************************************************************
 // Name:  char *GetSMErrorText
 // Input:   Error index returned by the menber functions
 // Output:  pointer to a string containing the error description
 // Description:
 // Author/Date:  jcar 20/9/96
 // History:
-//********************************************************************
+//*******************************************************************/
 char *GetSMErrorText(int index)
 {
 
@@ -181,7 +184,7 @@ char *GetSMErrorText(int index)
 }
 
 
-//********************************************************************
+/*********************************************************************
 // Name:  TSendText
 // Input:       1) RPath:   return path of the message
 //                                  Is used to fill the "Return-Path" and the
@@ -194,16 +197,14 @@ char *GetSMErrorText(int index)
 // Description:
 // Author/Date:  jcar 20/9/96
 // History:
-//********************************************************************
+//*******************************************************************/
 int SendText(char *RPath, char *Subject, char *mailTo, char *data, char *headers)
 {
-
 	int res, i;
 	char *p;
 	char *tempMailTo, *token, *pos1, *pos2;
-	TLS_VARS;
 
-	// check for NULL parameters
+	/* check for NULL parameters */
 	if (data == NULL)
 		return (BAD_MSG_CONTENTS);
 	if (mailTo == NULL)
@@ -211,44 +212,45 @@ int SendText(char *RPath, char *Subject, char *mailTo, char *data, char *headers
 	if (RPath == NULL)
 		return (BAD_MSG_RPATH);
 
-	// simple checks for the mailto address
-	// have ampersand ?
+	/* simple checks for the mailto address */
+	/* have ampersand ? */
 	if (strchr(mailTo, '@') == NULL)
 		return (BAD_MSG_DESTINATION);
 
-	sprintf(GLOBAL(Buffer), "HELO %s\r\n", GLOBAL(LocalHost));
+	sprintf(Buffer, "HELO %s\r\n", LocalHost);
 
-	// in the beggining of the dialog
-	// attempt reconnect if the first Post fail
-	if ((res = Post(GLOBAL(Buffer))) != SUCCESS) {
+	/* in the beggining of the dialog */
+	/* attempt reconnect if the first Post fail */
+	if ((res = Post(Buffer)) != SUCCESS) {
 		MailConnect();
-		if ((res = Post(GLOBAL(Buffer))) != SUCCESS)
+		if ((res = Post(Buffer)) != SUCCESS)
 			return (res);
 	}
 	if ((res = Ack()) != SUCCESS)
 		return (res);
 
-	sprintf(GLOBAL(Buffer), "MAIL FROM:<%s>\r\n", RPath);
-	if ((res = Post(GLOBAL(Buffer))) != SUCCESS)
+	sprintf(Buffer, "MAIL FROM:<%s>\r\n", RPath);
+	if ((res = Post(Buffer)) != SUCCESS)
 		return (res);
 	if ((res = Ack()) != SUCCESS)
 		return (res);
 
+
 	tempMailTo = estrdup(mailTo);
 	
-	// Send mail to all rcpt's
+	/* Send mail to all rcpt's */
 	token = strtok(tempMailTo, ",");
 	while(token != NULL)
 	{
-		sprintf(GLOBAL(Buffer), "RCPT TO:<%s>\r\n", token);
-		if ((res = Post(GLOBAL(Buffer))) != SUCCESS)
+		sprintf(Buffer, "RCPT TO:<%s>\r\n", token);
+		if ((res = Post(Buffer)) != SUCCESS)
 			return (res);
 		if ((res = Ack()) != SUCCESS)
 			return (res);
 		token = strtok(NULL, ",");
 	}
 
-	// Send mail to all Cc rcpt's
+	/* Send mail to all Cc rcpt's */
 	efree(tempMailTo);
 	if (headers && (pos1 = strstr(headers, "Cc:"))) {
 		pos2 = strstr(pos1, "\r\n");
@@ -257,8 +259,8 @@ int SendText(char *RPath, char *Subject, char *mailTo, char *data, char *headers
 		token = strtok(tempMailTo, ",");
 		while(token != NULL)
 		{
-			sprintf(GLOBAL(Buffer), "RCPT TO:<%s>\r\n", token);
-			if ((res = Post(GLOBAL(Buffer))) != SUCCESS)
+			sprintf(Buffer, "RCPT TO:<%s>\r\n", token);
+			if ((res = Post(Buffer)) != SUCCESS)
 				return (res);
 			if ((res = Ack()) != SUCCESS)
 				return (res);
@@ -273,7 +275,7 @@ int SendText(char *RPath, char *Subject, char *mailTo, char *data, char *headers
 		return (res);
 
 
-	// send message header
+	/* send message header */
 	if (Subject == NULL)
 		res = PostHeader(RPath, "No Subject", mailTo, headers);
 	else
@@ -282,7 +284,7 @@ int SendText(char *RPath, char *Subject, char *mailTo, char *data, char *headers
 		return (res);
 
 
-	// send message contents in 1024 chunks
+	/* send message contents in 1024 chunks */
 	if (strlen(data) <= 1024) {
 		if ((res = Post(data)) != SUCCESS)
 			return (res);
@@ -296,18 +298,18 @@ int SendText(char *RPath, char *Subject, char *mailTo, char *data, char *headers
 			else
 				i = strlen(p);
 
-			// put next chunk in buffer
-			strncpy(GLOBAL(Buffer), p, i);
-			GLOBAL(Buffer)[i] = '\0';
+			/* put next chunk in buffer */
+			strncpy(Buffer, p, i);
+			Buffer[i] = '\0';
 			p += i;
 
-			// send chunk
-			if ((res = Post(GLOBAL(Buffer))) != SUCCESS)
+			/* send chunk */
+			if ((res = Post(Buffer)) != SUCCESS)
 				return (res);
 		}
 	}
 
-	//send termination dot
+	/*send termination dot */
 	if ((res = Post("\r\n.\r\n")) != SUCCESS)
 		return (res);
 	if ((res = Ack()) != SUCCESS)
@@ -318,7 +320,7 @@ int SendText(char *RPath, char *Subject, char *mailTo, char *data, char *headers
 
 
 
-//********************************************************************
+/*********************************************************************
 // Name:  PostHeader
 // Input:       1) return path
 //                  2) Subject
@@ -328,21 +330,20 @@ int SendText(char *RPath, char *Subject, char *mailTo, char *data, char *headers
 // Description:
 // Author/Date:  jcar 20/9/96
 // History:
-//********************************************************************
+//********************************************************************/
 int PostHeader(char *RPath, char *Subject, char *mailTo, char *xheaders)
 {
 
-	// Print message header according to RFC 822
-	// Return-path, Received, Date, From, Subject, Sender, To, cc
+	/* Print message header according to RFC 822 */
+	/* Return-path, Received, Date, From, Subject, Sender, To, cc */
 
 	time_t tNow = time(NULL);
 	struct tm *tm = localtime(&tNow);
 	int zoneh = abs(_timezone);
 	int zonem, res;
 	char *p;
-	TLS_VARS;
 
-	p = GLOBAL(Buffer);
+	p = Buffer;
 	zoneh /= (60 * 60);
 	zonem = (abs(_timezone) / 60) - (zoneh * 60);
 
@@ -371,7 +372,7 @@ int PostHeader(char *RPath, char *Subject, char *mailTo, char *xheaders)
 		p += sprintf(p, "%s\r\n", xheaders);
 	}
 
-	if ((res = Post(GLOBAL(Buffer))) != SUCCESS)
+	if ((res = Post(Buffer)) != SUCCESS)
 		return (res);
 
 	if ((res = Post("\r\n")) != SUCCESS)
@@ -382,44 +383,50 @@ int PostHeader(char *RPath, char *Subject, char *mailTo, char *xheaders)
 
 
 
-//********************************************************************
+/*********************************************************************
 // Name:  MailConnect
 // Input:   None
 // Output:  None
 // Description: Connect to the mail host and receive the welcome message.
 // Author/Date:  jcar 20/9/96
 // History:
-//********************************************************************
+//********************************************************************/
 int MailConnect()
 {
 
 	int res;
-	TLS_VARS;
+	short portnum;
 
-
-	// Create Socket
-	if ((GLOBAL(sc) = socket(PF_INET, SOCK_STREAM, 0)) == INVALID_SOCKET)
+	/* Create Socket */
+	if ((sc = socket(PF_INET, SOCK_STREAM, 0)) == INVALID_SOCKET)
 		return (FAILED_TO_OBTAIN_SOCKET_HANDLE);
 
-	// Get our own host name
-	if (gethostname(GLOBAL(LocalHost), HOST_NAME_LEN))
+	/* Get our own host name */
+	if (gethostname(LocalHost, HOST_NAME_LEN))
 		return (FAILED_TO_GET_HOSTNAME);
 
-	// Resolve the servers IP
-	//if (!isdigit(GLOBAL(MailHost)[0])||!gethostbyname(GLOBAL(MailHost)))
-	//{
-	//	return (FAILED_TO_RESOLVE_HOST);
-	//}
+	/* Resolve the servers IP */
+	/*
+	if (!isdigit(MailHost[0])||!gethostbyname(MailHost))
+	{
+		return (FAILED_TO_RESOLVE_HOST);
+	}
+        */
 
-	// Connect to server
-	GLOBAL(sock_in).sin_family = AF_INET;
-	GLOBAL(sock_in).sin_port = htons(25);
-	GLOBAL(sock_in).sin_addr.S_un.S_addr = GetAddr(GLOBAL(MailHost));
+	portnum = (short) INI_INT("sendmail_port");
+	if (!portnum) {
+		portnum = 25;
+	}
 
-	if (connect(GLOBAL(sc), (LPSOCKADDR) & GLOBAL(sock_in), sizeof(GLOBAL(sock_in))))
+	/* Connect to server */
+	sock_in.sin_family = AF_INET;
+	sock_in.sin_port = htons(portnum);
+	sock_in.sin_addr.S_un.S_addr = GetAddr(MailHost);
+
+	if (connect(sc, (LPSOCKADDR) & sock_in, sizeof(sock_in)))
 		return (FAILED_TO_CONNECT);
 
-	// receive Server welcome message
+	/* receive Server welcome message */
 	res = Ack();
 	return (res);
 }
@@ -429,23 +436,22 @@ int MailConnect()
 
 
 
-//********************************************************************
+/*********************************************************************
 // Name:  Post
 // Input:
 // Output:
 // Description:
 // Author/Date:  jcar 20/9/96
 // History:
-//********************************************************************
+//********************************************************************/
 int Post(LPCSTR msg)
 {
 	int len = strlen(msg);
 	int slen;
 	int index = 0;
-	TLS_VARS;
 
 	while (len > 0) {
-		if ((slen = send(GLOBAL(sc), msg + index, len, 0)) < 1)
+		if ((slen = send(sc, msg + index, len, 0)) < 1)
 			return (FAILED_TO_SEND);
 		len -= slen;
 		index += slen;
@@ -455,7 +461,7 @@ int Post(LPCSTR msg)
 
 
 
-//********************************************************************
+/*********************************************************************
 // Name:  Ack
 // Input:
 // Output:
@@ -464,14 +470,13 @@ int Post(LPCSTR msg)
 // last command was successful.
 // Author/Date:  jcar 20/9/96
 // History:
-//********************************************************************
+//********************************************************************/
 int Ack()
 {
 	static char *buf;
 	int rlen;
 	int Index = 0;
 	int Received = 0;
-	TLS_VARS;
 
 	if (!buf)
 		if ((buf = (char *) malloc(1024 * 4)) == NULL)
@@ -479,18 +484,18 @@ int Ack()
 
   again:
 
-	if ((rlen = recv(GLOBAL(sc), buf + Index, ((1024 * 4) - 1) - Received, 0)) < 1)
+	if ((rlen = recv(sc, buf + Index, ((1024 * 4) - 1) - Received, 0)) < 1)
 		return (FAILED_TO_RECEIVE);
 
 	Received += rlen;
 	buf[Received] = 0;
-	//err_msg   fprintf(stderr,"Received: (%d bytes) %s", rlen, buf + Index);
+	/*err_msg   fprintf(stderr,"Received: (%d bytes) %s", rlen, buf + Index); */
 
-	// Check for newline
+	/* Check for newline */
 	Index += rlen;
 	if ((buf[Received - 2] != '\r') || (buf[Received - 1] != '\n'))
-		// err_msg          fprintf(stderr,"Incomplete server message. Awaiting CRLF\n");
-		goto again;				// Incomplete data. Line must be terminated by CRLF
+		/* err_msg          fprintf(stderr,"Incomplete server message. Awaiting CRLF\n"); */
+		goto again;				/* Incomplete data. Line must be terminated by CRLF */
 
 	if (buf[0] > '3')
 		return (SMTP_SERVER_ERROR);
@@ -499,7 +504,7 @@ int Ack()
 }
 
 
-//********************************************************************
+/*********************************************************************
 // Name:  unsigned long GetAddr (LPSTR szHost)
 // Input:
 // Output:
@@ -510,7 +515,7 @@ int Ack()
 // WARNING: gethostbyname() is a blocking function
 // Author/Date:  jcar 20/9/96
 // History:
-//********************************************************************
+//********************************************************************/
 unsigned long GetAddr(LPSTR szHost)
 {
 	LPHOSTENT lpstHost;
